package interface_graphique;
import java.awt.FlowLayout;

import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JRadioButton;

import common.FullDuplexMsgWorker;

/**
 * 
 * @author Éric Lallet.
 *
 */
public class InterfaceGraphique extends JFrame  {
	
	/**
	 * 
	 */
	private static final long serialVersionUID = -1174002294163719760L;
	/**
	 * Longueur de la fenêtre de saisie du texte.
	 */
	static final int LINESIZE = 100;
	
	/**
	 * Nombre de propositions de réponses.
	 */
	static final int NBPROPOSITIONS = 4;
	
	/**
	 * JPanel de SWING qui contiendra le texte du tchat.
	 */
	private Texte texte;
	
	/**
	 * JButton du bouton pour valider la réponse.
	 */
	private JButton boutonValider; 
	/**
	 * radio boutons des 4 propositions A/B/C/D.
	 */
	private JRadioButton[] boutonsRadio = new  JRadioButton[NBPROPOSITIONS];
	/**
	 * passe à faux quand une réponse a été validée. Remis à vrai à la question suivante.
	 */
	private boolean reponsePossible = false;
	





	/**
	 * Pseudonyme de l'utilisateur.
	 */
	private String pseudo;
	/**
	 * worker pour la communication TCP avec la serveur.
	 */
	private FullDuplexMsgWorker worker;
	
	/**
	 * 
	 * @param nbLignes
	 *                nombre de lignes de la fenêtre d'affichage du tchat.
	 * @param pseudo
	 *              pseudonyme de l'utilisateur.
	 * @param worker
	 *              worker pour la communication TCP avec la serveur.
	 */
	public InterfaceGraphique(final int nbLignes, final String pseudo, final FullDuplexMsgWorker worker) {
		this.pseudo = pseudo;
		this.worker = worker;
		build(nbLignes, pseudo);
	}

	
	/**
	 * Création de des fenêtres de l'interface graphique. 
	 *  -> La fenêtre principale associée à la JFrame de cette classe
	 *  -> Avec juste un dessin de dedans (JPanel de la classe Dessin)
	 * @param nbLignes
	 *                nombre de lignes de la fenêtre d'affichage du tchat.
	 * @param pseudo
	 *              pseudonyme de l'utilisateur.
	 */
	private void build(final int nbLignes, final String pseudo) {
		
		
		// Titre de la fenêtre principale
		this.setTitle(pseudo); 
		
		// Disposition du contenu de la fenêtre principale
		this.setLayout(new FlowLayout());
		
		// On centre la fenêtre sur l'écran
		this.setLocationRelativeTo(null); 
		
		// On interdit la redimensionnement de la fenêtre
		this.setResizable(false);
		
		// On dit à l'application de se fermer lors du clic
		// sur la croix
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		
		// Ajout du contenu de la fenêtre
		this.setContentPane(buildContentPane(nbLignes));
		
		// cette instruction "réalise" tout ce qui a été préparé avant.
		this.pack();

	}

	/**
	 * 
	 * @param nbLignes
	 *                nombre de lignes de la fenêtre d'affichage du tchat.
	 * @return le JPanel SWING qui contiendra le texte du tchat
	 */
	private JPanel buildContentPane(final int nbLignes) {
		JPanel panel = new JPanel();
		JPanel boutonPanel = new JPanel();
		JPanel ligneBasPanel = new JPanel();
		JPanel hautPanel = new JPanel();
		ButtonGroup boutonGroup = new ButtonGroup();
		 panel.setLayout(new BoxLayout(panel, BoxLayout.PAGE_AXIS));
		
		 // Les lignes de texte
	    this.texte = new Texte(nbLignes);
	    this.texte.setAlignmentX(JPanel.LEFT_ALIGNMENT);
	    
	    
	    
	    // puis la ligne de saisie
	   // this.saisie = new JTextField(LINESIZE);
	   // this.saisie.setAlignmentX(JTextField.LEFT_ALIGNMENT);
	    
	    
	    boutonPanel = new JPanel();
	    char lettre = 'A';
	    for (int ind = 0; ind < NBPROPOSITIONS; ind++) {
	    	String nomBouton = "" + lettre;
	    	this.boutonsRadio[ind] = new JRadioButton(nomBouton);
	    	boutonGroup.add(this.boutonsRadio[ind]);
	    	boutonPanel.add(this.boutonsRadio[ind]);
	    	lettre++;
	    }
	    
	    boutonValider = new JButton("Valider");
	    ligneBasPanel.add(boutonPanel);
	    ligneBasPanel.add(boutonValider);
	    hautPanel.add(this.texte);
	    panel.add(hautPanel);
	    panel.add(ligneBasPanel);
	   
		
	
		return panel;
	}

	/**
	 * 
	 * @return le JPnanel SWING contenant le texte du tchat.
	 */
	public Texte getTexte() {
		return this.texte;
	}
	

	
	/**
	 * 
	 * @return le pseudonyme de l'utilisateur.
	 */
	public String getPseudo() {
		return this.pseudo;
	}
	
	/**
	 * 
	 * @return le worker pour la communication TCP avec la serveur.
	 */
	public FullDuplexMsgWorker getWorker() {
		return this.worker;
	}
	
	/**
	 * 
	 * @return
	 *        les radio boutons des quatre propositions.
	 */
	public JRadioButton[] getBoutonsRadio() {
		return boutonsRadio;
	}

	/**
	 * 
	 * @return
	 *        JButtun du bouton pour valider une réponse.
	 */
	public JButton getBoutonValider() {
		return boutonValider;
	}

	/**
	 * 
	 * @return
	 *        faux quand une réponse a déjà été validée,  vrai pour une nouvelle question.
	 */
	public boolean isReponsePossible() {
		return reponsePossible;
	}

	/**
	 * 
	 * @param reponsePossible
	 *           vrai qd il est possible de valider une réponse, faux sinon.
	 */
	public void setReponsePossible(final boolean reponsePossible) {
		this.reponsePossible = reponsePossible;
		this.boutonValider.setEnabled(reponsePossible);
	}



	

}
