package interface_graphique;

import java.awt.Dimension;
import java.util.Vector;
import javax.swing.JPanel;
import javax.swing.BoxLayout;
import javax.swing.JLabel;

/**
 * 
 * @author Éric Lallet.
 *
 */
public class Texte extends JPanel {
	/**
	 * largeur du JPanel du texte.
	 */
	static final int LARGEURPANEL = 800;
	
	/**
	 * hauteur du JPanel du texte.
	 */
	//static final int HAUTEURPANEL = 800;
	
	/**
	 * pixels verticaux par ligne de texte.
	 */
	static final int PIXELPERLINE = 17;
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 4596452846823469466L;
	/**
	 * les lignes contenants le texte du tchat.
	 */
	private Vector<JLabel> leTexte = new Vector<JLabel>();
	/**
	 *  le nombre de lignes du tchat.
	 */
	private final int nbLignes; 
	
	
	/**
	 * 
	 * @param nbLignes
	 *                le nombre de lignes du tchat.
	 */
	public Texte(final int nbLignes) {
		super();
		int hauteurPanel = nbLignes * PIXELPERLINE;
		this.setPreferredSize(new Dimension(LARGEURPANEL, hauteurPanel));
		this.setLayout(new BoxLayout(this, BoxLayout.PAGE_AXIS));
		this.nbLignes = nbLignes;
		for (int ind = 0; ind < nbLignes; ind++) {
			JLabel label = new JLabel(); 
			label.setAlignmentX(JLabel.LEFT_ALIGNMENT);
			leTexte.addElement(label);
			this.add(label);
		}
		
	}
	
	

	/**
	 * 
	 * @param ligne
	 *             ligne à ajouter au texte du tchat.
	 */
	public void ecrireNouvelleLigne(final String ligne) {
		/*
		 * Une solution pour gérer l'accès concurentiel au Vector par 
		 * 2 threads différents est de synchroniser la section critique.
		 * 
		 * Pour le cas particulier de SWING la vraie bonne solution est
		 * en commentaire dans la classe TheadClient du paquetage tcpnio
		 * Elle consiste à faire en sorte que seul le Thread de SWING
		 * appelle cette méthode.
		 *  
		 * Mais vous trouverez ci dessous, en commentaire, une façon
		 * de synchroniser une section critique.  
		 */
		//synchronized (leTexte) {
			for (int ind = 0; ind < this.nbLignes - 1; ind++) {
				this.leTexte.elementAt(ind).setText(this.leTexte.elementAt(ind + 1).getText());
				this.leTexte.elementAt(ind).repaint();
			}
			
			this.leTexte.elementAt(this.nbLignes - 1).setText(ligne);
			this.leTexte.elementAt(this.nbLignes - 1).repaint();
		//}
		this.repaint();

	}
		
	/**
	 * ajoute plusieurs lignes à la fenètre de texte.
	 * @param texte
	 *            texte à écrire. Peut contenir plusieurs lignes.
	 */
	public void ecrireNouveauTexte(final String texte) {
		String[] lignes = texte.split("[\\r\\n]");
		for (String uneLigne: lignes) {
			this.ecrireNouvelleLigne(uneLigne);
		}
	}
	
}
