package json;

/**
 * This class has the same structure that the json response of the web service.
 * Needed for parsing the json message. 
 * @author Éric Lallet.
 *
 */
public class Response {
	/**
	 * json response code (1st json field of the response of the web service).
	 */
	private int response_code;
	
	/**
	 * json results: list of results (the web service can send several questions in the response). 
	 */
	private Result[] results;
	
	@Override
	public String toString() {
		String res =  "{\"response_code\":" + response_code + ",\"results\":[";
		//+ results + "]}";

		for (int ind = 0; ind < results.length - 1; ind++) {
			res = res + "\"" + results[ind] + "\",";
		}
		if (results.length > 0) {
			res = res + "\"" + results[results.length - 1] + "\""; 
		}
		res = res + "]}";
		
		return res; 
	}
	
	/**
	 * 
	 * @return
	 *        categoty of the 1st result.
	 */
	public String getCategory() {
		return results[0].getCategory();
	}
	
	/**
	 * 
	 * @return
	 *        difficulty of the 1st result.
	 */
	public String getDifficulty() {
		return results[0].getDifficulty();
	}
	
	/**
	 * 
	 * @return
	 *         question of the 1st result.
	 */
	public String getQuestion() {
		return results[0].getQuestion();
	}
	
	/**
	 * 
	 * @return
	 *        correct answer of the 1st result.
	 */
	public String getCorrectAnswer() {
		return results[0].getCorrectAnswer();
	}
	
	/**
	 * 
	 * @param num
	 *           position in the list of the incorrect propositions.
	 * @return
	 *         incorrect proposition at the num position of the 1st result.
	 */
	public String getIncorrectAnswer(final int num) {
		return results[0].getIncorrectAnswer(num);
	}

}
