package trivial;

import static common.Log.LOGGER_NAME_COMM;

import java.io.IOException;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.SocketException;
import java.net.UnknownHostException;

import java.nio.channels.SocketChannel;
import java.lang.reflect.InvocationTargetException;

import javax.swing.SwingUtilities;

import org.apache.log4j.Level;

import common.FullDuplexMsgWorker;
import common.Log;
import interface_graphique.CallBack;
import interface_graphique.CreationInterfaceGraphique;
import interface_graphique.InterfaceGraphique;



/**
 * 
 * @author Éric Lallet.
 *
 */
public final class AppliClient {
	/**
	 * number of expected arguments of main method.
	 */
	private static final int NBARGS = 3;
	/**
	 * position of nickname in argv[].
	 */
	private static  final int PSEUDOARG = 0;	
	/**
	 * position of the hostname in argv[].
	 */
	private static  final int HOSTARG = 1;
	/**
	 * position of the port number in argv[].
	 */
	private static  final int PORTARG = 2;
	/**
	 * 
	 */
	/**
	 * number of lines of the chat texte.
	 */
	private static final int NBLIGNES = 40;

	/**
	 * 
	 */
	private AppliClient() {
		
	}
	/**
	 * 
	 * @param argv
	 *            argv[0] user nickname.
	 *            argv[1] server hostanme.
	 *            argv[2] server port number.
	 */
	public static void main(final String[] argv) {
		/**
		 * worker for TCP communication with the server.
		 */
		FullDuplexMsgWorker worker = null;
		/**
		 * channel connected to the server.
		 */
		SocketChannel rwChan;

		/**
		 * SWING Thread for the creation of the user interface.
		 */
		CreationInterfaceGraphique creationInterfaceGraphique = null; // Tâche envoyée au thread de Swing
		
		/**
		 * SWING JFrame of the user interface.
		 */
		InterfaceGraphique interfaceGraphique;

		if (argv.length != NBARGS) {
			System.out.println("usage: java MainMsgNbClient <pseudo> <machine> <port> ");
			return;
		}

		Log.configureALogger(LOGGER_NAME_COMM, Level.TRACE); // tout est logué pour la comm

		try {
			InetAddress destAddr = InetAddress.getByName(argv[HOSTARG]);
			rwChan = SocketChannel.open(new InetSocketAddress(destAddr, Integer.parseInt(argv[PORTARG])));
			worker = new FullDuplexMsgWorker(rwChan);
			


		} catch (SocketException se) {
			se.printStackTrace();
		} catch (UnknownHostException ue) {
			ue.printStackTrace();
		} catch (IOException ie) {
			ie.printStackTrace();
		} 
		
		

		
		
		// --------Construction de l'interface graphique -----// 
		// On envoie la construction de l'interface au Thread de Swing. 
		// On attend que tout soit prêt pour
		// pouvoir placer les CallBacks ensuite
		try {
			creationInterfaceGraphique = new CreationInterfaceGraphique(NBLIGNES, argv[PSEUDOARG], worker);
			SwingUtilities.invokeAndWait(creationInterfaceGraphique);
		} catch (InvocationTargetException e1) {
			e1.printStackTrace();
		} catch (InterruptedException e1) {
			e1.printStackTrace();
		}
		interfaceGraphique = creationInterfaceGraphique.getFenetre();
		

		// On lance Thread Client chargé de lire les messages du serveur
		new Thread(new ThreadClient(worker, interfaceGraphique)).start();

		// ---------Mise en place du CallBack chargé de lire les messages des utilisateurs
		// fait dans le Thread de Swing
		new CallBack(interfaceGraphique);
		
		// Fin du Thread principal: il reste 2 Threads en vie:
		//  - celui en charge de la lecture des messages du serveur
		//  - celui de Swing en charge de la lecture des messages des utilisisateurs
		
	}
}
