package trivial;

import static common.Log.LOGGER_NAME_TEST;
import static common.Log.LOG_ON;
import static common.Log.TEST;
import static org.junit.Assert.assertNotEquals;
import static org.junit.Assert.assertTrue;

import java.io.IOException;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.nio.channels.SocketChannel;



import org.apache.log4j.Level;
import org.junit.Ignore;
import org.junit.Test;

import common.FullDuplexMsgWorker;
import common.Log;
import common.ReadMessageStatus;
import json.TrivialQuestion;


public class TestChangeState {
	static public final int PORT = 5010;
	static public final int PORT2 = 5020;
	static public final String HOST = "localhost";
	static public final String NICK = "Pseudo";



	//@Ignore
	@Test
	public void testChangeState() throws IOException {
		Log.configureALogger(LOGGER_NAME_TEST, Level.TRACE);
		if (LOG_ON && TEST.isInfoEnabled()) {
			TEST.info("starting " + TestId.CHANGESTATE);
		}

		if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
			TEST.trace("testing changeStateAndSendData: 1st pass ");
		}	

		changeStatePass(PORT, TestId.CHANGESTATE3);

		if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
			TEST.trace("testing changeStateAndSendData: 2nd pass ");
		}	

		changeStatePass(PORT2, TestId.CHANGESTATE5);

		if (LOG_ON && TEST.isInfoEnabled()) {
			TEST.info("end of " + TestId.CHANGESTATE);
		}
	}

	private void changeStatePass(final int port, final TestId testId) throws IOException {
		if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
			TEST.trace("testing changeStateAndSendData: creating server");
		}	
		// create new server
		TrivialServer trivialServer = new TrivialServer(port, AppliServer.TRIVIALHTTPURL, TestId.CHANGESTATE);

		// ************************************************
		// ****** expected server state: Sleeping *********

		if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
			TEST.trace("testing changeStateAndSendData: creating ServerSocketChannel");
		}	
		// server: open the listen channel
		trivialServer.playTest(TestId.OPENLISTENCHANNEL);

		if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
			TEST.trace("testing changeStateAndSendData: connect to server");
		}	
		// connecting to the server
		SocketChannel  rwChan = SocketChannel.open(new InetSocketAddress(InetAddress.getByName(HOST),port ));
		FullDuplexMsgWorker worker = new FullDuplexMsgWorker(rwChan);

		if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
			TEST.trace("testing changeStateAndSendData: accept");
		}	
		// server: accept new player
		trivialServer.playTest(TestId.ACCEPTNEWPLAYER);



		/*********************** TEST  CHANGESTATE1 *****************************/
		if (LOG_ON && TEST.isInfoEnabled()) {
			TEST.info("starting " + TestId.CHANGESTATE1);
		}

		if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
			TEST.trace("testing changeStateAndSendData: change to waiting test");
		}	
		// server: changeState Sleeping->Waiting
		trivialServer.playTest(TestId.CHANGESTATE1);

		// ************************************************
		// ****** expected server state: Waiting  *********

		// receiving data

		if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
			TEST.trace("testing changeStateAndSendData: should receive a new question");
		}	

		ReadMessageStatus status = ReadMessageStatus.ReadUnstarted;
		while ((status != ReadMessageStatus.ChannelClosed) && (status != ReadMessageStatus.ReadDataCompleted)) { 
			status = worker.readMessage();
		}

		if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
			TEST.trace("testing changeStateAndSendData: check data");
		}	
		assertTrue("Testting data completed" , status == ReadMessageStatus.ReadDataCompleted);

		TrivialQuestion trivialQuestion = (TrivialQuestion) worker.getData();


		/*********************** TEST  CHANGESTATE2 *****************************/
		if (LOG_ON && TEST.isInfoEnabled()) {
			TEST.info("starting " + TestId.CHANGESTATE2);
		}

		if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
			TEST.trace("testing changeStateAndSendData: change time to swicht to Pending state");
		}
		// server: changeState Waiting -> Pending
		trivialServer.playTest(TestId.CHANGESTATE2);

		// ************************************************
		// ****** expected server state: Pending  *********

		if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
			TEST.trace("testing changeStateAndSendData: should receive an answersListn");
		}	

		status = ReadMessageStatus.ReadUnstarted;
		while ((status != ReadMessageStatus.ChannelClosed) && (status != ReadMessageStatus.ReadDataCompleted)) { 
			status = worker.readMessage();
		}

		if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
			TEST.trace("testing changeStateAndSendData: check data");
		}
		assertTrue("Testting data completed" , status == ReadMessageStatus.ReadDataCompleted);

		assertTrue("is instance of answerList ", worker.getData() instanceof AnswersList );

		switch (testId) {
		case CHANGESTATE3:
			// ************************************************
			// ****** expected server state: Pending  *********

			/*********************** TEST  CHANGESTATE3 *****************************/
			if (LOG_ON && TEST.isInfoEnabled()) {
				TEST.info("starting " + TestId.CHANGESTATE3);
			}

			if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
				TEST.trace("testing changeStateAndSendData: change time to swicht to Waiting state");
			}
			// server: changeState Pending -> Waiting
			trivialServer.playTest(TestId.CHANGESTATE3);	

			// ************************************************
			// ****** expected server state: Waiting  *********

			if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
				TEST.trace("testing changeStateAndSendData: should receive a new question");
			}

			status = ReadMessageStatus.ReadUnstarted;
			while ((status != ReadMessageStatus.ChannelClosed) && (status != ReadMessageStatus.ReadDataCompleted)) { 
				status = worker.readMessage();
			}

			if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
				TEST.trace("testing changeStateAndSendData: check data");
			}
			assertTrue("Testting data completed" , status == ReadMessageStatus.ReadDataCompleted);

			trivialQuestion = (TrivialQuestion) worker.getData();


			/*********************** TEST  CHANGESTATE4 *****************************/
			if (LOG_ON && TEST.isInfoEnabled()) {
				TEST.info("starting " + TestId.CHANGESTATE4);
			}
			if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
				TEST.trace("testing changeStateAndSendData: disconnect the client -> should switch to Sleeping State");
			}

			worker.close();

			// server: changeState Waiting -> Sleeping
			trivialServer.playTest(TestId.CHANGESTATE4);

			// ************************************************
			// ****** expected server state: Sleeping  *********

			if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
				TEST.trace("testing changeStateAndSendData: connect a new client");
			}
			// connecting to the server
			rwChan = SocketChannel.open(new InetSocketAddress(InetAddress.getByName(HOST),port ));
			worker = new FullDuplexMsgWorker(rwChan);

			if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
				TEST.trace("testing changeStateAndSendData: accept new client");
			}	
			// server: accept new player
			trivialServer.playTest(TestId.ACCEPTNEWPLAYER);


			if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
				TEST.trace("testing changeStateAndSendData: should change to waiting State");
			}	
			// server: changeState Sleeping->Waiting
			trivialServer.playTest(TestId.CHANGESTATE1);


			// ************************************************
			// ****** expected server state: Waiting  *********

			if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
				TEST.trace("testing recieving new question");
			}

			if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
				TEST.trace("testing changeStateAndSendData: should receive a new question");
			}
			status = ReadMessageStatus.ReadUnstarted;
			while ((status != ReadMessageStatus.ChannelClosed) && (status != ReadMessageStatus.ReadDataCompleted)) { 
				status = worker.readMessage();
			}

			if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
				TEST.trace("testing changeStateAndSendData: check data");
			}
			assertTrue("Testting data completed" , status == ReadMessageStatus.ReadDataCompleted);

			trivialQuestion = (TrivialQuestion) worker.getData();
			break;
		case CHANGESTATE5:
			// ************************************************
			// ****** expected server state: Pending  *********

			if (LOG_ON && TEST.isInfoEnabled()) {
				TEST.info("starting " + TestId.CHANGESTATE5);
			}
			if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
				TEST.trace("testing changeStateAndSendData: disconnect the client -> should switch to Sleeping State");
			}

			worker.close();
			// server: changeState Pending -> Sleeping
			trivialServer.playTest(TestId.CHANGESTATE5);

			// ************************************************
			// ****** expected server state: Sleeping  *********

			if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
				TEST.trace("testing changeStateAndSendData: connect a new client");
			}
			// connecting to the server
			rwChan = SocketChannel.open(new InetSocketAddress(InetAddress.getByName(HOST),port ));
			worker = new FullDuplexMsgWorker(rwChan);

			if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
				TEST.trace("testing changeStateAndSendData: accept new client");
			}	
			// server: accept new player
			trivialServer.playTest(TestId.ACCEPTNEWPLAYER);


			if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
				TEST.trace("testing changeStateAndSendData: should change to waiting State");
			}	
			// server: changeState Sleeping->Waiting
			trivialServer.playTest(TestId.CHANGESTATE1);
			// ************************************************
			// ****** expected server state: Waiting  *********


			if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
				TEST.trace("testing recieving new question");
			}



			status = ReadMessageStatus.ReadUnstarted;
			while ((status != ReadMessageStatus.ChannelClosed) && (status != ReadMessageStatus.ReadDataCompleted)) { 
				status = worker.readMessage();
			}

			if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
				TEST.trace("testing changeStateAndSendData: check data");
			}
			assertTrue("Testting data completed" , status == ReadMessageStatus.ReadDataCompleted);

			if (LOG_ON && TEST.isEnabledFor(Level.TRACE)) {
				TEST.trace("testing changeStateAndSendData: previus question and new question should be different");
			}

			TrivialQuestion trivialQuestion2 = (TrivialQuestion) worker.getData();
			assertNotEquals("the 2 questions are not equals", trivialQuestion, trivialQuestion2);
			break;
		default:

		}


	}



}
